import express from 'express';
import request from 'request';
import jwt from 'express-jwt';
import mappings from './api-mapping.json';
import config from './config';
import handleUnauthorizedRequest from './middlewares/handleUnauthorizedRequest';
import enableCors from './middlewares/enableCors';

const app = express();
const appId = 'pingit';

// offers the function for registering a route and redirecting it to the provided server
function registerAppMethod(host, port, basePath, redirect) {
    const { path, method, auth, cors } = redirect;
    const routePath = basePath + path;

    function getExtendedHeaders(req) {
        const headers = {...req.headers, ...{ 'appid': appId}};
        // host header must be set as undefined, otherwise
        // provokes issues with https calls
        headers.host = undefined;
        return headers;
    }

    function handleRequest(req, res) {

        const pathTo = req.originalUrl.slice(basePath.length);
        const uri = host + ':' + port + pathTo;

        console.log('[INFO] API request on %s:%s%s send to %s:%s%s', server.address().address, server.address().port, req.originalUrl, host, port, pathTo);

        const rreq = request({
            uri,
            method,
            body: req.body,
            headers: getExtendedHeaders(req)
        });
        req.pipe(rreq).pipe(res);
    }

    const middlewares = [];
    if (auth) {
        middlewares.push(jwt({secret: config.authSecret}));
    }
    if (cors) {
        middlewares.push(enableCors);
    }

    switch (method) {
        case 'GET':
            app.get(routePath, middlewares, handleRequest);
            break;
        case 'POST':
            app.post(routePath, middlewares, handleRequest);
            break;
        case 'DELETE':
            app.delete(routePath, middlewares, handleRequest);
            break;
        case 'PUT':
            app.put(routePath, middlewares, handleRequest);
            break;
        case 'OPTIONS':
            app.options(routePath, middlewares, handleRequest);
            break;
        default:
            console.warn('method ' + method + ' it is not supported');
    }
}

mappings.forEach((mapping) => {
    const { basePath, host, port, redirects} = mapping;
    redirects.forEach((redirect) => {
        const { path, method, auth} = redirect;
        registerAppMethod(host, port, basePath, redirect);
        console.log('[INIT] Created route to %s %s:%s%s - auth[%s]', method, host, port, path, auth === true);
    });
});

app.use(handleUnauthorizedRequest);

// starts the server
const server = app.listen(9000, function() {
    const host = server.address().address;
    const port = server.address().port;
    console.log('[INFO] listening at http://%s:%s', host, port);
});
